require("AtlasHelpers");

local rootLogger = "AtlasSystems.Addons.CCCGetItNowDSPAddon";

luanet.load_assembly("System");
luanet.load_assembly("mscorlib");
luanet.load_assembly("log4net");

local Types = {};
Types["System.Net.WebClient"] = luanet.import_type("System.Net.WebClient");
Types["System.Text.Encoding"] = luanet.import_type("System.Text.Encoding");
Types["log4net.LogManager"] = luanet.import_type("log4net.LogManager");

local log = Types["log4net.LogManager"].GetLogger(rootLogger);

-- Load settings
local Settings = {};
Settings.GetItNowURL = GetSetting("GetItNowURL");
Settings.AutomationDecisionRouteQueue = GetSetting("AutomationDecisionRouteQueue");

function Init()
    Settings.GetItNowURL = string.gsub(Settings.GetItNowURL, "[/ ]*$", "");
    Settings.AutomationDecisionRouteQueue = AtlasHelpers.Trim(Settings.AutomationDecisionRouteQueue);

    RegisterSystemEventHandler("GenerateFulfillmentData", "LookupBarcode");
    RegisterSystemEventHandler("AutomationDecision", "RouteTransaction");
end

function LookupBarcode(eventArgs)
    local tn = GetFieldValue("Transaction", "TransactionNumber");    
    local issn = GetFieldValue("Transaction", "ISSN");
    if not issn or issn == "" then
        ExecuteCommand("AddNote", {tn, "Fulfillment data could not be generated because the request is missing an ISSN."});
        eventArgs.Handled = false;
        eventArgs.Available = false;
        return;
    end
    issn = AtlasHelpers.Trim(issn);

    local year = GetFieldValue("Transaction", "PhotoJournalYear");
    if year:len() > 4 then
        year=year:match("%d%d%d%d")
    end

    log:DebugFormat("Looking up ISSN \"{0}\" for Transaction: {1}", issn, tn);

    local response = HttpGetRequest(
        Settings.GetItNowURL .. "/request/info?" ..
        "issn=" .. AtlasHelpers.UrlEncode(issn) ..
        "&year=" .. AtlasHelpers.UrlEncode(year) ..
        "&partner=ILLIAD"
    );

    if string.upper(AtlasHelpers.Trim(response)) == "N" then
        log:InfoFormat("ISSN \"{0}\" is not available.", issn);
        eventArgs.Handled = true;
        eventArgs.Available = false;
        return
    end

    log:DebugFormat("Response from GetItNow: {0}", response);

    local parsedResponse = AtlasHelpers.StringSplit(",", response);
    for index, term in ipairs(parsedResponse) do
        parsedResponse[index] = AtlasHelpers.Trim(term);
    end

    if #parsedResponse ~= 3 then
        local message = "The response from CCC GetItNow does not have the right number of terms. This addon requires exactly 3 terms. Respose: " .. response;
        log:Error(message);
        error({ Message = message });
    end

    local availableFlag = string.upper(parsedResponse[1]);
    local cost = tonumber(parsedResponse[2]);
    local publisher = parsedResponse[3];

    log:DebugFormat("Creating a fulfillment option for Transaction: {0} (Available? {1})", tn, availableFlag);

    eventArgs.Available = availableFlag == "Y";
    eventArgs.Cost = cost;
    eventArgs.TurnaroundTime = 0;
    eventArgs.CopyrightIncluded = true;
    eventArgs.Notes = 'Publisher: ' .. publisher;

    eventArgs.Handled = true;
end

function RouteTransaction(eventArgs)
    local tn = GetFieldValue("Transaction", "TransactionNumber");

    if Settings.AutomationDecisionRouteQueue ~= "" then
        log:DebugFormat("Routing Transaction {0} to \"{1}\"", tn, Settings.AutomationDecisionRouteQueue);
        ExecuteCommand("Route", {tn, Settings.AutomationDecisionRouteQueue, false});
        eventArgs.Handled = true;
        return;
    end

    log:WarnFormat(
        "This addon is not configured to route Transactions. Transaction {0} will be handled by the Decision Support AutomationDecision Stage's default error handling.",
        tn
    );
end

function HttpGetRequest(uri)
    log:DebugFormat("HTTP GET Request URI: {0}.", uri);

    local client = Types["System.Net.WebClient"]();
    client.Encoding = Types["System.Text.Encoding"].UTF8;

    client.Headers:Clear();
    client.Headers:Add("User-Agent", "ILLiad-Addon");

    log:Debug("Sending GET request");

    local success, result = pcall(function()
        return client:DownloadString(uri);
    end);

    client:Dispose();

    if (success) then
        log:Debug("GET request sent successfully");
        return result;
    end

    local errorMessage = GetErrorMessage(result);
    log:ErrorFormat("Error sending request. {0}", errorMessage);
    error({ Message = errorMessage });
end

function GetErrorMessage(e)
    log:Debug("Getting error message");
    if not e.Message then
        -- Not a .NET exception
        log:Debug("Not a .NET exception");
        return tostring(e);
    end

    if e.InnerException then
        return e.Message .. "\r\n" .. GetErrorMessage(e.InnerException);
    else
        return e.Message;
    end
end
